# makeshift hack of AI::MXNet::Module

package Makeshift::MXNet::Module;

use Mouse;
use AI::MXNet;
use AI::MXNet::Module;
use Encode;
use JSON;

extends 'AI::MXNet::Module';

has 'current_epoch' => (is => 'ro', writer => '_current_epoch',
						default => 0);
has 'info' => (is => 'ro', writer => '_info', lazy_build => 1);

around 'fit' => sub {
	my $orig = shift;
	my $self = shift;
	my $train_data = shift;
	my %arg = @_;

	$arg{begin_epoch} //= $self->current_epoch;
	$self->_current_epoch($arg{begin_epoch} + $arg{num_epoch});
	$self->$orig($train_data, %arg);
};

__PACKAGE__->meta->make_immutable();

sub add_epoch {
	my $self = next;
	my $num = next;
	my $epoch = $self->current_epoch;

	$self->_current_epoch($epoch + $num);

	return $self->current_epoch;
}

sub save_current {
	my $self = shift;
	my $prefix = shift;
	my $epoch = $self->current_epoch;

	$self->save_checkpoint($prefix, $epoch);
	my $json = to_json($self->info);
	$json = encode_utf8($json) if Encode::is_utf8($json);

	my $info_file = "$prefix-info.json";
	open(INFO, ">", $info_file) or die "$!";
	print INFO $json;
	close(INFO);
}

sub load_current {
	my $class = shift;
	my $prefix = shift;
	my $info_file = "$prefix-info.json";

	open(INFO, "<", $info_file) or die "$!";
	my $json = do {local $/; <INFO>};
	close(INFO);
	$json = decode_utf8($json) unless Encode::is_utf8($json);
	my $info = from_json($json);

	my $batch_size = shift // $info->{batch_size} // 1;

	${$info->{data_shapes}[0]->[1]}[0] = $batch_size;
	${$info->{label_shapes}[0]->[1]}[0] = $batch_size;

	my $model = $class->load($prefix, $info->{current_epoch});

	$model->bind(data_shapes  => $info->{data_shapes},
				 label_shapes => $info->{label_shapes},
				 for_training => 0);

	$model->_current_epoch($info->{current_epoch});
	$model->_info($info);

	return $model;
}

sub add_info {
	my $self = shift;
	my %arg = @_;

	for (keys %arg) {
		$self->info->{$_} = $arg{$_};
	}

	return $self->info;
}

sub _build_info {
	my $self = shift;
	my %info = (current_epoch => $self->current_epoch,
				data_shapes => [$self->data_shapes->[0]->to_nameshape],
				label_shapes => [$self->label_shapes->[0]->to_nameshape]);

	return \%info;
}


1;
# end of package
