#!/usr/bin/perl
use lib "$ENV{HOME}/lib/perl5";
use strict;
use warnings;
use PDL;
use Math::Random::Discrete;
use AI::MXNet qw(mx);
use AI::MXNet::Function::Parameters;
use Makeshift::MXNet::Module;
use Getopt::Long qw(HelpMessage);

GetOptions(
    'prefix=s'       => \(my $prefix  = 'lstm' ),
#    'sample-size=i'  => \(my $sample_size  = 10000   ),
    'sample-size=i'  => \(my $sample_size  = 200   ),
    'help'           => sub { HelpMessage(0) },
) or HelpMessage(1);

=head1 NAME

    char_lstm_gen.pl - Example of generating char LSTM RNN on tiny shakespeare using high level RNN interface

=head1 SYNOPSIS

    --prefix         prefix for saved files, default='lstm'
    --sample-size    a size of inferred sample text (default=200)

=cut


my $model = Makeshift::MXNet::Module->load_current($prefix);
my $seq_size = $model->info->{seq_size};
my $batch_size = $model->info->{batch_size};
my $fdata = pdl($model->info->{fdata});
my %vocabulary = %{$model->info->{vocabulary}};
my %reverse_vocab = reverse %vocabulary;

sample();
print "\n";

sub sample {
    return if not $sample_size;
    $model->reshape(data_shapes=>[['data',[1, $seq_size]]], label_shapes=>[['softmax_label',[1, $seq_size]]]);
    my $input = mx->nd->array($fdata->slice([0, $seq_size-1]))->reshape([1, $seq_size]);
    my $label = mx->nd->array($fdata->slice([1, $seq_size]))->reshape([1, $seq_size]);
    $| = 1;
    for (0..$sample_size-1)
    {
#        $model->forward(mx->io->DataBatch(data=>[$input]), is_train => 0);
        $model->forward(mx->io->DataBatch(data=>[$input], label=>[$label]), is_train => 0);
#
        my $prob = $model->get_outputs(0)->[0][0]->at($seq_size-1)->aspdl;
        my $next_char = Math::Random::Discrete->new($prob->reshape(-1)->unpdl, [0..scalar(keys %vocabulary)-1])->rand;
        print "$reverse_vocab{$next_char}";
        $input->at(0)->slice([0, $seq_size-2]) .= $input->at(0)->slice([1, $seq_size-1])->copy;
        $input->at(0)->at($seq_size-1) .= $next_char;
    }
    $model->reshape(data_shapes=>[['data',[$batch_size, $seq_size]]], label_shapes=>[['softmax_label',[$batch_size, $seq_size]]]);
}

# end of script
